﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing;

namespace UltimatumGame
{
    class Drawer
    {
        protected PictureBox screen;
        protected int sizeX, sizeY;

        protected Graphics g;

        public int granularity;
        protected int bucketSize;

        protected GameMaster game;

        protected long maxCount;
        protected double minScore, maxScore;

        protected Color lowScoreColor = Color.Red, highScoreColor = Color.Green;

        protected virtual Color getColor(double score)
        {
            if (score < 0)
                return Color.LightGray;
            if (maxScore == minScore)
                return lowScoreColor;

            int red = (int)(((highScoreColor.R - lowScoreColor.R) * (score - minScore)) / (maxScore - minScore) + lowScoreColor.R);
            int blue = (int)(((highScoreColor.B - lowScoreColor.B) * (score - minScore)) / (maxScore - minScore) + lowScoreColor.B);
            int green = (int)(((highScoreColor.G - lowScoreColor.G) * (score - minScore)) / (maxScore - minScore) + lowScoreColor.G);
            Color result = Color.FromArgb(red, green, blue);
            return result;
        }

        protected double[,] ScoreInBuckets, OfferingInBuckets, AcceptanceInBuckets;
        protected int[,] countInBucket, RoundsInBucket;

        protected void computeScores()
        {
            for (int i = 0; i < granularity; i++)
            {
                for (int j = 0; j < granularity; j++)
                {
                    ScoreInBuckets[i, j] = 0;
                    countInBucket[i, j] = 0;
                    OfferingInBuckets[i, j] = 0;
                    AcceptanceInBuckets[i, j] = 0;
                    RoundsInBucket[i, j] = 0;
                }
            }

            maxScore = -1;
            minScore = int.MaxValue;

            foreach (var item in game.players)
            {
                int bucketX = (int)(item.offerPercentage * granularity);
                if (bucketX >= granularity - 1)
                    bucketX = granularity - 1;
                int bucketY = (int)(item.acceptancePercentage * granularity);
                if (bucketY >= granularity - 1)
                    bucketY = granularity - 1;

                ScoreInBuckets[bucketX, bucketY] += item.score;
                countInBucket[bucketX, bucketY]++;
                OfferingInBuckets[bucketX, bucketY] += item.offerPercentage;
                AcceptanceInBuckets[bucketX, bucketY] += item.acceptancePercentage;
                RoundsInBucket[bucketX, bucketY] += item.roundsPlayed;
            }
            maxCount = 0;
            for (int i = 0; i < granularity; i++)
            {
                for (int j = 0; j < granularity; j++)
                {
                    double score = RoundsInBucket[i, j] == 0 ? ScoreInBuckets[i, j] : ScoreInBuckets[i, j] / RoundsInBucket[i, j];
                    if (maxScore < score)
                        maxScore = score;
                    if (minScore > score)
                        minScore = score;
                    if (maxCount < countInBucket[i, j])
                        maxCount = countInBucket[i, j];
                }
            }
        }
        public virtual void draw()
        {
            bucketSize = Math.Min(sizeX / (granularity + 1), sizeY / granularity);
            g.Clear(Color.White);
            computeScores();
            for (int i = 0; i < granularity; i++)
            {
                for (int j = 0; j < granularity; j++)
                {
                    g.FillRectangle(new SolidBrush(getColor(getValue(i,j))), i * bucketSize, j * bucketSize, bucketSize, bucketSize);
                    g.DrawString(countInBucket[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize));
                    g.DrawString(RoundsInBucket[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize) + 20, (float)(j * bucketSize));
                    g.DrawString("Off: " + (OfferingInBuckets[i, j]/countInBucket[i,j]).ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize) + 20);
                    g.DrawString("Acc: " + (AcceptanceInBuckets[i, j]/countInBucket[i,j]).ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize) + 40);
                    g.DrawString("Score: " + (ScoreInBuckets[i, j] / countInBucket[i, j]).ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize) + 60);
                }
            }
            drawGrid();
            screen.Refresh();
        }

        public virtual double getValue(int x, int y)
        {
            return RoundsInBucket[x, y] == 0 ? -1 : (int)Math.Floor(ScoreInBuckets[x, y] / RoundsInBucket[x, y]);
        }

        Pen LinePen = Pens.Black;

        protected void drawGrid()
        {
            for (int i = 0; i <= granularity; i++)
            {
                g.DrawLine(LinePen, i * bucketSize, 0, i * bucketSize, bucketSize * granularity);
                g.DrawLine(LinePen, 0, i * bucketSize, bucketSize * granularity, i * bucketSize);
            }
        }

        public Drawer(PictureBox pic, GameMaster gm, int granularity)
        {
            this.screen = pic;
            this.sizeX = pic.Width;
            this.sizeY = pic.Height;
            pic.Image = new Bitmap(sizeX, sizeY);
            this.granularity = granularity;
            this.game = gm;
            this.g = Graphics.FromImage(pic.Image);
            this.ScoreInBuckets = new double[granularity, granularity];
            this.AcceptanceInBuckets = new double[granularity, granularity];
            this.OfferingInBuckets = new double[granularity, granularity];
            this.countInBucket = new int[granularity, granularity];
            this.RoundsInBucket = new int[granularity, granularity];
        }

        public Drawer()
        {
        }

        public virtual double getMaxValue()
        {
            return maxScore; 
        }

        public virtual double getMinValue()
        {
            return minScore;
        }

        public void create3DGraph(string outputFileName)
        {
            string name = "JMENO";
            using (var writer = new System.IO.StreamWriter(@"..\Graph\" + outputFileName))
            {
                writer.Write(@"<!DOCTYPE html>
<html lang=""en""><head>
<meta http-equiv=""content-type"" content=""text/html; charset=UTF-8"">
    <title>");
                writer.Write(name);
                writer.Write(@"</title>
    <meta charset=""utf-8"">
    <link rel=""stylesheet"" href=""reset.css"">
<link rel=""stylesheet"" href=""scenes.css"">
  </head>
  
  <body style=""background-color: rgb(100, 100, 100);"">
    
    <div id=""valuelabel"">
</div>

<div id=""div-legend"">
  <h4>");
                writer.Write(name + "</h4>");
                writer.Write(@"  
</div>

    <div id=""preview-footer"">
      Created with <a href=""http://threegraphs.com/"" target=""_blank"">threegraphs.com</a>. Powererd by <a href=""http://mrdoob.github.com/three.js/"" target=""_blank"">Three.js</a>
    </div> 
   
  <script src=""ga.js"" async="""" type=""text/javascript""></script><script type=""text/javascript"">
    chartType = ""bar"";
    loadType = ""sample"";
  </script>
  
  <script type=""text/javascript"" src=""settings.js""></script>
  <script type=""text/javascript"">
    staticUrl = ""/static/"";
    chartTitle = """ + name + @"""
  </script>
  ");

    writer.Write(@"  <script type=""text/javascript"">schema =
    {
        ""cols"": [");

                for (int i = 0; i < granularity; i++)
                {
                    string colName = "Offers " + (int)Math.Floor((double)i*100/granularity) + "%";
                    //string colName = "AOH";
                    if (i % 2 == 0)
                        writer.Write(@"{ ""name"": """ + colName + @""", ""color"": ""fc55f6"" }");
                    else writer.Write(@"{ ""name"": """ + colName + @""", ""color"": ""2b6bdf"" }");
                    //ae00e6 fc55f6 2b6bdf 83cf63
                    if (i < granularity - 1)
                        writer.WriteLine(",");
                }

    writer.WriteLine(@"],
        ""rows"": [");
                
                for (int i = 0; i < granularity; i++)
                {
                    string rowName = "Accepts " + (int)Math.Floor((double)i * 100 / granularity) + "%";
                    writer.Write(@" { ""name"": """ + rowName + @""" }");
                    if (i < granularity - 1)
                        writer.WriteLine(",");
                }

   writer.WriteLine(@"]
}; dataValues = [");
   for (int i = 0; i < granularity; i++)
   {
       writer.Write("[");
       for (int j = 0; j < granularity; j++)
       {
           writer.Write(getValue(j,i));
           if (j < granularity - 1)
               writer.Write(",");
           else writer.Write("]");
       }
       if (i < granularity - 1)
           writer.WriteLine(","); 
   }
   writer.WriteLine(@"];
   </script>");

                writer.Write(@"  <script type=""text/javascript"" src=""jquery-1.js""></script>
  <script type=""text/javascript"" src=""three.js""></script>
  <script type=""text/javascript"" src=""TrackballControls.js""></script>
  <script type=""text/javascript"" src=""TrackballControlsTouch.js""></script>
  <script type=""text/javascript"" src=""utils.js""></script>
  <script type=""text/javascript"" src=""NiceScale.js""></script>
  <script type=""text/javascript"" src=""BarCubes.js""></script>
  <script type=""text/javascript"" src=""ScaleTexts.js""></script>
  <script type=""text/javascript"" src=""helvetiker_bold.js""></script>
  <script type=""text/javascript"" src=""scene_bar.js""></script>
  <script type=""text/javascript"">
    
    valHeight=1000;backColor='000000';scaleTextColor='ffffff';pieHeight=150;countryFocus=""Libya"";valTextColor='ffffff';
    
  </script>
  <script type=""text/javascript"" src=""init_embed.js""></script><canvas height=""943"" width=""1920""></canvas>
  

</body></html>");

            }
        }
    }

    class CountsDrawer : Drawer
    {
        protected Color lowNumberColor = Color.Gold, highNumberColor = Color.Blue;

        public CountsDrawer(PictureBox pictureBox2, GameMaster gm, int p)
            : base(pictureBox2, gm, p)
        {

        }

        public override double getValue(int x, int y)
        {
            return countInBucket[x, y];
        }

        public override double getMaxValue()
        {
            return maxCount;
        }

        public override double getMinValue()
        {
            return 0;
        }

        protected Color getColorNumbers(int number)
        {
            int minScore = 0;
            if (number <= 0)
                return Color.LightGray;
            if (maxCount == minScore)
                return lowNumberColor;

            int red = (int)(((highNumberColor.R - lowNumberColor.R) * (number - minScore)) / (maxCount - minScore) + lowNumberColor.R);
            int blue = (int)(((highNumberColor.B - lowNumberColor.B) * (number - minScore)) / (maxCount - minScore) + lowNumberColor.B);
            int green = (int)(((highNumberColor.G - lowNumberColor.G) * (number - minScore)) / (maxCount - minScore) + lowNumberColor.G);
            Color result = Color.FromArgb(red, green, blue);
            return result;
        }

        public override void draw()
        {
            bucketSize = Math.Min(sizeX / (granularity + 1), sizeY / granularity);
            g.Clear(Color.White);
            computeScores();
            for (int i = 0; i < granularity; i++)
            {
                for (int j = 0; j < granularity; j++)
                {
                    g.FillRectangle(new SolidBrush(getColorNumbers((int)getValue(i,j))), i * bucketSize, j * bucketSize, bucketSize, bucketSize);
                    g.DrawString(countInBucket[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize));
                    /*
                    g.DrawString(RoundsInBucket[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize) + 20, (float)(j * bucketSize));
                    g.DrawString("Off: " + OfferingInBuckets[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize) + 20);
                    g.DrawString("Acc: " + AcceptanceInBuckets[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize) + 40);
                    g.DrawString("Score: " + ScoreInBuckets[i, j].ToString(), new Font("Arial", 10), Brushes.Black, (float)(i * bucketSize), (float)(j * bucketSize) + 60);
                     */
                }
            }
            //drawGrid();
            screen.Refresh();
        }

    }

}
